@@ -17,6 +17,18 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  */
 
+/**
+ * SECTION:gdict-client-context
+ * @short_description: DICT client transport
+ *
+ * #GdictClientContext is an implementation of the #GdictContext interface.
+ * It implements the Dictionary Protocol as defined by the RFC 2229 in order
+ * to connect to a dictionary server.
+ *
+ * You should rarely instantiate this object directely: use an appropriate
+ * #GdictSource instead.
+ */
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -799,6 +811,7 @@ gdict_client_context_run_command (GdictClientContext  *context,
                                   GError             **error)
 {
   GdictClientContextPrivate *priv;
+  gchar *payload;
   GError *send_error;
   gboolean res;
   
@@ -828,9 +841,11 @@ gdict_client_context_run_command (GdictClientContext  *context,
   switch (command->cmd_type)
     {
     case CMD_CLIENT:
-      command->cmd_string = g_strdup_printf ("%s \"%s\"\r\n",
+      payload = g_shell_quote (priv->client_name);
+      command->cmd_string = g_strdup_printf ("%s %s\r\n",
                                              dict_command_strings[CMD_CLIENT],
-                                             priv->client_name);
+                                             payload);
+      g_free (payload);
       break;
     case CMD_QUIT:
       command->cmd_string = g_strdup_printf ("%s\r\n",
@@ -846,18 +861,22 @@ gdict_client_context_run_command (GdictClientContext  *context,
       break;
     case CMD_MATCH:
       g_assert (command->word);
-      command->cmd_string = g_strdup_printf ("%s %s %s \"%s\"\r\n",
+      payload = g_shell_quote (command->word);
+      command->cmd_string = g_strdup_printf ("%s %s %s %s\r\n",
                                              dict_command_strings[CMD_MATCH],
                                              (command->database != NULL ? command->database : "!"),
                                              (command->strategy != NULL ? command->strategy : "*"),
-                                             command->word);
+                                             payload);
+      g_free (payload);
       break;
     case CMD_DEFINE:
       g_assert (command->word);
-      command->cmd_string = g_strdup_printf ("%s %s \"%s\"\r\n",
+      payload = g_shell_quote (command->word);
+      command->cmd_string = g_strdup_printf ("%s %s %s\r\n",
                                              dict_command_strings[CMD_DEFINE],
                                              (command->database != NULL ? command->database : "!"),
-                                             command->word);
+                                             payload);
+      g_free (payload);
       break;
     default:
       g_assert_not_reached ();
@@ -903,11 +922,14 @@ clear_command_queue (GdictClientContext *context)
 {
   GdictClientContextPrivate *priv = context->priv;
 
-  g_queue_foreach (priv->commands_queue,
-                   (GFunc) gdict_command_free,
-                   NULL);
+  if (priv->commands_queue)
+    {
+      g_queue_foreach (priv->commands_queue,
+                       (GFunc) gdict_command_free,
+                       NULL);
   
-  g_queue_free (priv->commands_queue);
+      g_queue_free (priv->commands_queue);
+    }
   
   /* renew */
   priv->commands_queue = g_queue_new ();
@@ -932,6 +954,12 @@ gdict_client_context_force_disconnect (GdictClientContext *context)
           
       priv->channel = NULL;
     }
+
+  if (priv->command)
+    {
+      gdict_command_free (priv->command);
+      priv->command = NULL;
+    }
   
   clear_command_queue (context);
 }
@@ -1817,8 +1845,10 @@ check_for_connection (gpointer data)
 {
   GdictClientContext *context = data;
 
+#if 0
   g_debug (G_STRLOC ": checking for connection (is connecting:%s)",
            context->priv->is_connecting ? "true" : "false");
+#endif
 
   if (context->priv->is_connecting)
     {
@@ -2092,7 +2122,7 @@ gdict_client_context_define_word (GdictContext  *context,
   
   cmd = gdict_command_new (CMD_DEFINE);
   cmd->database = g_strdup ((database != NULL ? database : GDICT_DEFAULT_DATABASE));
-  cmd->word = g_strdup (word);
+  cmd->word = g_utf8_normalize (word, -1, G_NORMALIZE_NFC);
   
   return gdict_client_context_push_command (client_ctx, cmd);
 }
@@ -2127,7 +2157,7 @@ gdict_client_context_match_word (GdictContext  *context,
   cmd = gdict_command_new (CMD_MATCH);
   cmd->database = g_strdup ((database != NULL ? database : GDICT_DEFAULT_DATABASE));
   cmd->strategy = g_strdup ((strategy != NULL ? strategy : GDICT_DEFAULT_STRATEGY));
-  cmd->word = g_strdup (word);
+  cmd->word = g_utf8_normalize (word, -1, G_NORMALIZE_NFC);
   
   return gdict_client_context_push_command (client_ctx, cmd);
 }
